MODULE agrif_lim3_interp
   !!=====================================================================================
   !!                       ***  MODULE agrif_lim3_interp ***
   !! Nesting module :  interp surface ocean boundary condition over ice from a parent grid
   !! Sea-Ice model  :  LIM 3.6 Sea ice model time-stepping
   !!=====================================================================================
   !! History :  2.0   !  04-2008  (F. Dupont)  initial version
   !!            3.4   !  09-2012  (R. Benshila, C. Herbaut) update and EVP
   !!            3.6   !  05-2016  (C. Rousset)  Add LIM3 compatibility
   !!----------------------------------------------------------------------
#if defined key_agrif && defined key_lim3 
   !!----------------------------------------------------------------------
   !!   'key_lim3'  :                                 LIM 3.6 sea-ice model
   !!   'key_agrif' :                                 AGRIF library
   !!----------------------------------------------------------------------
   !!  agrif_interp_lim3    : interpolation of ice at "after" sea-ice time step
   !!  agrif_interp_u_ice   : atomic routine to interpolate u_ice 
   !!  agrif_interp_v_ice   : atomic routine to interpolate v_ice 
   !!  agrif_interp_tra_ice : atomic routine to interpolate ice properties 
   !!----------------------------------------------------------------------
   USE par_oce
   USE dom_oce
   USE sbc_oce
   USE ice
   USE agrif_ice

   IMPLICIT NONE
   PRIVATE

   PUBLIC agrif_interp_lim3

   !!----------------------------------------------------------------------
   !! NEMO/NST 3.6 , NEMO Consortium (2016)
   !! $Id: agrif_lim3_interp.F90 6204 2016-01-04 13:47:06Z cetlod $
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE agrif_interp_lim3( cd_type )
      !!-----------------------------------------------------------------------
      !!                 *** ROUTINE agrif_rhg_lim3  ***
      !!
      !!  ** Method  : simple call to atomic routines using stored values to
      !!  fill the boundaries depending of the position of the point and 
      !!  computing factor for time interpolation 
      !!-----------------------------------------------------------------------
      CHARACTER(len=1), INTENT( in ) :: cd_type
      !!    
      REAL(wp) :: zbeta
      !!-----------------------------------------------------------------------
      !
      IF( Agrif_Root() )  RETURN
      !
      zbeta = REAL(lim_nbstep) / ( Agrif_Rhot() * REAL(Agrif_Parent(nn_fsbc)) / REAL(nn_fsbc) )
      !
      ! clem: calledweight = zbeta(1/3;2/3;1) => 2/3*var1+1/3*var2 puis 1/3;2/3 puis 0;1
      Agrif_SpecialValue=-9999.
      Agrif_UseSpecialValue = .TRUE.
      SELECT CASE(cd_type)
      CASE('U')
         CALL Agrif_Bc_variable( u_ice_id  , procname=interp_u_ice  , calledweight=zbeta )
      CASE('V')
         CALL Agrif_Bc_variable( v_ice_id  , procname=interp_v_ice  , calledweight=zbeta )
      CASE('T')
         CALL Agrif_Bc_variable( tra_ice_id, procname=interp_tra_ice, calledweight=zbeta )
      END SELECT
      Agrif_SpecialValue=0.
      Agrif_UseSpecialValue = .FALSE.
      !
   END SUBROUTINE agrif_interp_lim3

   !!------------------
   !! Local subroutines
   !!------------------
   SUBROUTINE interp_u_ice( ptab, i1, i2, j1, j2, before )
      !!-----------------------------------------------------------------------
      !!                     *** ROUTINE interp_u_ice ***
      !!
      !! i1 i2 j1 j2 are the index of the boundaries parent(when before) and child (when after)
      !! To solve issues when parent grid is "land" masked but not all the corresponding child grid points,
      !! put -9999 WHERE the parent grid is masked. The child solution will be found in the 9(?) points around
      !!-----------------------------------------------------------------------
      INTEGER , INTENT(in) :: i1, i2, j1, j2
      REAL(wp), DIMENSION(i1:i2,j1:j2), INTENT(inout) :: ptab
      LOGICAL , INTENT(in) :: before
      !!
      REAL(wp) :: zrhoy
      !!-----------------------------------------------------------------------
      !
      IF( before ) THEN  ! parent grid
         ptab(:,:) = e2u(i1:i2,j1:j2) * u_ice_b(i1:i2,j1:j2)
         WHERE( umask(i1:i2,j1:j2,1) == 0. )  ptab(:,:) = -9999.
      ELSE               ! child grid
         zrhoy = Agrif_Rhoy()
         u_ice(i1:i2,j1:j2) = ptab(:,:) / ( e2u(i1:i2,j1:j2) * zrhoy ) * umask(i1:i2,j1:j2,1)
      ENDIF
      !
   END SUBROUTINE interp_u_ice


   SUBROUTINE interp_v_ice( ptab, i1, i2, j1, j2, before )
      !!-----------------------------------------------------------------------
      !!                    *** ROUTINE interp_v_ice ***
      !!
      !! i1 i2 j1 j2 are the index of the boundaries parent(when before) and child (when after)
      !! To solve issues when parent grid is "land" masked but not all the corresponding child grid points,
      !! put -9999 WHERE the parent grid is masked. The child solution will be found in the 9(?) points around
      !!-----------------------------------------------------------------------      
      INTEGER , INTENT(in) :: i1, i2, j1, j2
      REAL(wp), DIMENSION(i1:i2,j1:j2), INTENT(inout) :: ptab
      LOGICAL , INTENT(in) :: before
      !!
      REAL(wp) :: zrhox
      !!-----------------------------------------------------------------------
      !
      IF( before ) THEN  ! parent grid
         ptab(:,:) = e1v(i1:i2,j1:j2) * v_ice_b(i1:i2,j1:j2)
         WHERE( vmask(i1:i2,j1:j2,1) == 0. )  ptab(:,:) = -9999.
      ELSE               ! child grid
         zrhox = Agrif_Rhox()
         v_ice(i1:i2,j1:j2) = ptab(:,:) / ( e1v(i1:i2,j1:j2) * zrhox ) * vmask(i1:i2,j1:j2,1)
      ENDIF
      !
   END SUBROUTINE interp_v_ice


   SUBROUTINE interp_tra_ice( ptab, i1, i2, j1, j2, k1, k2, before )
      !!-----------------------------------------------------------------------
      !!                    *** ROUTINE interp_tra_ice ***                           
      !!
      !! i1 i2 j1 j2 are the index of the boundaries parent(when before) and child (when after)
      !! To solve issues when parent grid is "land" masked but not all the corresponding child grid points,
      !! put -9999 WHERE the parent grid is masked. The child solution will be found in the 9(?) points around
      !!-----------------------------------------------------------------------
      REAL(wp), DIMENSION(i1:i2,j1:j2,k1:k2), INTENT(inout) :: ptab
      INTEGER , INTENT(in) :: i1, i2, j1, j2, k1, k2
      LOGICAL , INTENT(in) :: before
      !!
      INTEGER :: jk, jl, jm
      !!-----------------------------------------------------------------------
      ! clem: pkoi on n'utilise pas les quantités intégrées ici => before: * e12t ; after: * r1_e12t / rhox / rhoy
      ! a priori c'est ok comme ca (cf ce qui est fait dans l'ocean). Je ne sais pas pkoi ceci dit
      
      IF( before ) THEN  ! parent grid
         jm = 1
         DO jl = 1, jpl
            ptab(:,:,jm) = a_i_b  (i1:i2,j1:j2,jl) ; jm = jm + 1
            ptab(:,:,jm) = v_i_b  (i1:i2,j1:j2,jl) ; jm = jm + 1
            ptab(:,:,jm) = v_s_b  (i1:i2,j1:j2,jl) ; jm = jm + 1
            ptab(:,:,jm) = smv_i_b(i1:i2,j1:j2,jl) ; jm = jm + 1
            ptab(:,:,jm) = oa_i_b (i1:i2,j1:j2,jl) ; jm = jm + 1
            DO jk = 1, nlay_s
               ptab(:,:,jm) = e_s_b(i1:i2,j1:j2,jk,jl) ; jm = jm + 1
            ENDDO
            DO jk = 1, nlay_i
               ptab(:,:,jm) = e_i_b(i1:i2,j1:j2,jk,jl) ; jm = jm + 1
            ENDDO
         ENDDO
         !!ptab(:,:,jm) = ato_i(i1:i2,j1:j2)
         
         DO jk = k1, k2
            WHERE( tmask(i1:i2,j1:j2,1) == 0. )  ptab(:,:,jk) = -9999.
         ENDDO
         
      ELSE               ! child grid
         jm = 1
         DO jl = 1, jpl
            a_i  (i1:i2,j1:j2,jl) = ptab(:,:,jm) * tmask(i1:i2,j1:j2,1) ; jm = jm + 1
            v_i  (i1:i2,j1:j2,jl) = ptab(:,:,jm) * tmask(i1:i2,j1:j2,1) ; jm = jm + 1
            v_s  (i1:i2,j1:j2,jl) = ptab(:,:,jm) * tmask(i1:i2,j1:j2,1) ; jm = jm + 1
            smv_i(i1:i2,j1:j2,jl) = ptab(:,:,jm) * tmask(i1:i2,j1:j2,1) ; jm = jm + 1
            oa_i (i1:i2,j1:j2,jl) = ptab(:,:,jm) * tmask(i1:i2,j1:j2,1) ; jm = jm + 1
            DO jk = 1, nlay_s
               e_s(i1:i2,j1:j2,jk,jl) = ptab(:,:,jm) * tmask(i1:i2,j1:j2,1) ; jm = jm + 1
            ENDDO
            DO jk = 1, nlay_i
               e_i(i1:i2,j1:j2,jk,jl) = ptab(:,:,jm) * tmask(i1:i2,j1:j2,1) ; jm = jm + 1
            ENDDO
         ENDDO
         !!ato_i(i1:i2,j1:j2) = ptab(:,:,jm) * tmask(i1:i2,j1:j2,1)
         
      ENDIF
      !
   END SUBROUTINE interp_tra_ice

#else
CONTAINS
   SUBROUTINE agrif_lim3_interp_empty
      !!---------------------------------------------
      !!   *** ROUTINE agrif_lim3_interp_empty ***
      !!---------------------------------------------
      WRITE(*,*)  'agrif_lim3_interp : You should not have seen this print! error?'
   END SUBROUTINE agrif_lim3_interp_empty
#endif
END MODULE agrif_lim3_interp
